@@ -1,5 +1,5 @@
 # Copyright (C) 2005 Warp Networks S.L., DBS Servicios Informaticos S.L.
-# Copyright (C) 2007 Warp Networks S.L.
+# Copyright (C) 2007-2008 Warp Networks S.L.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2, as
@@ -20,27 +20,29 @@ use strict;
 use warnings;
 
 use base qw(
-            EBox::GConfModule 
-			EBox::Model::ModelProvider
-			EBox::Model::CompositeProvider
-			EBox::ServiceModule::ServiceInterface
-           ); 
+            EBox::GConfModule
+            EBox::Model::ModelProvider
+            EBox::ServiceModule::ServiceInterface
+           );
 
 # Interfaces list which will be ignored
 use constant ALLIFACES => qw(sit tun tap lo irda eth wlan vlan);
 use constant IGNOREIFACES => qw(sit tun tap lo irda);
 use constant IFNAMSIZ => 16; #Max length name for interfaces
+use constant INTERFACES_FILE => '/etc/network/interfaces';
 
 use Net::IP;
 use EBox::NetWrappers qw(:all);
 use EBox::Validate qw(:all);
 use EBox::Config;
 use EBox::Order;
+use EBox::ServiceModule::Manager;
 use EBox::Exceptions::InvalidData;
 use EBox::Exceptions::DataExists;
 use EBox::Exceptions::DataInUse;
 use EBox::Exceptions::Internal;
 use EBox::Exceptions::External;
+use EBox::Exceptions::MissingArgument;
 use Error qw(:try);
 use EBox::Summary::Composite;
 use EBox::Summary::Module;
@@ -53,8 +55,8 @@ use EBox::Gettext;
 #use EBox::LogAdmin qw( :all );
 use File::Basename;
 
-
-use EBox::Network::Report::ByteRate;
+# XXX uncomment when DynLoader bug with locales is fixed
+# use EBox::Network::Report::ByteRate;
 
 
 sub _create
@@ -94,6 +96,21 @@ sub actions
 	];
 }
 
+# Method: usedFiles
+#
+# 	Override EBox::ServiceModule::ServiceInterface::usedFiles
+#
+sub usedFiles
+{
+	return [ 
+	{	
+		'file' => INTERFACES_FILE,
+		'reason' => __('eBox will set your network configuration'),
+		'module' => 'network'
+	}
+	];
+}
+
 #  Method: serviceModuleName
 #
 #   Override EBox::ServiceModule::ServiceInterface::servivceModuleName
@@ -118,30 +135,84 @@ sub modelClasses
                           directory => 'multigwrulestable',
                          ],
           },
-          {
-           class      => 'EBox::Network::Model::ByteRateEnableForm',
-           parameters => [
-                          enableTitle  => __('Activate traffic rate monitor'),
-                          domain       => 'ebox-network',
-                          modelDomain  => 'Network',
-                         ],
-          },
-	  'EBox::Network::Model::ByteRateSettings',
-	  'EBox::Network::Model::ByteRateGraph',
-	  'EBox::Network::Model::ByteRateGraphControl',
+
+# XXX uncomment when DynLoader bug with locales is fixed
+ #          {
+#            class      => 'EBox::Network::Model::ByteRateEnableForm',
+#            parameters => [
+#                           enableTitle  => __('Activate traffic rate monitor'),
+#                           domain       => 'ebox-network',
+#                           modelDomain  => 'Network',
+#                          ],
+#           },
+# 	  'EBox::Network::Model::ByteRateSettings',
+# 	  'EBox::Network::Model::ByteRateGraph',
+# 	  'EBox::Network::Model::ByteRateGraphControl',
+          'EBox::Network::Model::StaticRoute',
+          'EBox::Network::Model::DeletedStaticRoute',
+          'EBox::Network::Model::DNSResolver',
 	 ];
 }
 
 
-
-sub compositeClasses
+# Method: _exposedMethods
+#
+#
+# Overrides:
+#
+#      <EBox::Model::ModelProvider::_exposedMethods>
+#
+# Returns:
+#
+#      hash ref - the list of the exposes method in a hash ref every
+#      component
+#
+sub _exposedMethods
 {
-  return [
-	  'EBox::Network::Composite::ByteRate',
-	 ];
-
+    my %exposedMethods =
+      (
+       'addRoute' => { action => 'add',
+                       path   => [ 'StaticRoute' ],
+                     },
+       'delRoute' => { action  => 'del',
+                       path    => [ 'StaticRoute' ],
+                       indexes => [ 'network' ],
+                       separator => ':',
+                   },
+       'changeGateway' => { action   => 'set',
+                            path     => [ 'StaticRoute' ],
+                            indexes  => [ 'network' ],
+                            selector => [ 'gateway' ],
+                            separator => ':'
+                          },
+       'addNS'    => { action => 'add',
+                       path   => [ 'DNSResolver' ],
+                     },
+       'setNS'    => { action   => 'set',
+                       path     => [ 'DNSResolver' ],
+                       indexes  => [ 'position' ],
+                       selector => [ 'nameserver' ],
+                     },
+       'removeNS' => { action  => 'del',
+                       path    => [ 'DNSResolver' ],
+                       indexes => [ 'position' ],
+                     },
+      );
+    return \%exposedMethods;
 }
 
+
+
+# XXX uncomment when DynLoader bug with locales is fixed
+#sub compositeClasses
+#{
+#  return [
+#	  'EBox::Network::Composite::ByteRate',
+#	 ];
+#
+#}
+
+
 # Method: IPAddressExists
 #
 # 	Returns true if the given IP address belongs to a statically configured
@@ -1353,7 +1424,8 @@ sub ifaceNetmask # (interface)
 		return $self->get_string("interfaces/$name/netmask");
 	} elsif ($self->ifaceMethod($name) eq 'dhcp') {
 		if (iface_is_up($name)) {
-			return iface_netmask($name);
+			#return iface_netmask($name);
+                    return (values %{iface_addresses_with_netmask($name)})[0]
 		}
 	}
 	return undef;
@@ -1425,70 +1497,102 @@ sub ifaceBroadcast # (interface)
 	return undef;
 }
 
-# Method: nameservers 
-#	
-#	Returns a list of the configured name servers
+# Method: nameservers
+#
+#	Return a list of the configured name servers
 #
 #  Returns:
 #
-#	an array ref - each element contains a string holding the nameserver
+#	Array ref - each element contains a string holding the nameserver
+#
 sub nameservers
 {
-	my $self = shift;
+	my ($self) = @_;
 	my @array = ();
-	foreach (1..2) {
-		my $server = $self->get_string("nameserver" . $_);
-		(defined($server) and ($server ne '')) or next;
-		push(@array, $server);
-	}
-	return \@array;
+# 	foreach (1..2) {
+# 		my $server = $self->get_string("nameserver" . $_);
+# 		(defined($server) and ($server ne '')) or next;
+# 		push(@array, $server);
+# 	}
+# 	return \@array;
+        my $resolverModel = $self->model('DNSResolver');
+        my $rows = $resolverModel->rows();
+        @array = map { $_->valueByName('nameserver') } @{$rows};
+        return \@array;
 }
 
 # Method: nameserverOne
-#	
-#	Returns the primary nameserver's IP address 
+#
+#	Return the primary nameserver's IP address
 #
 #  Returns:
 #
-#	string - nameserver's IP address	
+#	String - nameserver's IP address
+#
+#       empty string - if there is not primary nameserver
+#
 sub nameserverOne
 {
-	my $self = shift;
-	return $self->get_string("nameserver1");
+	my ($self) = @_;
+#	return $self->get_string("nameserver1");
+        my $nss = $self->nameservers();
+        return $nss->[0] if (scalar(@{$nss}) >= 1);
+        return '';
 }
 
 # Method: nameserverTwo
-#	
-#	Returns the secondary nameserver's IP address 
+#
+#	Return the secondary nameserver's IP address
 #
 #  Returns:
 #
-#	string - nameserver's IP address	
+#	string - nameserver's IP address
+#
+#       empty string - if there is not secondary nameserver
+#
 sub nameserverTwo
 {
 	my $self = shift;
-	return $self->get_string("nameserver2");
+#	return $self->get_string("nameserver2");
+        my $nss = $self->nameservers();
+        return $nss->[1] if (scalar(@{$nss}) >= 2);
+        return '';
 }
 
 # Method: setNameservers
-#	
-#	Sets the nameservers
 #
-#   Paramaters:
+#	Set a set of name server resolvers
 #
-#	one - primary nameserver
-#	two - secondary nameserver
-sub setNameservers # (one, two) 
+# Parameters:
+#
+#       array - a list of IP addresses which are the name server
+#       resolvers
+#
+sub setNameservers # (one, two)
 {
 	my ($self, @dns) = @_;
-	my @nameservers = ();
-	my $i = 0;
-	foreach (@dns) {
-		$i++;
-		($i < 3) or last;
-		(length($_) == 0) or checkIP($_, __("IP address"));
-		$self->set_string("nameserver$i", $_);
-	}
+# 	my @nameservers = ();
+# 	my $i = 0;
+# 	foreach (@dns) {
+# 		$i++;
+# 		($i < 3) or last;
+# 		(length($_) == 0) or checkIP($_, __("IP address"));
+# 		$self->set_string("nameserver$i", $_);
+# 	}
+        my $nss = $self->nameservers();
+        my $resolverModel = $self->model('DNSResolver');
+        my $nNSS = scalar(@{$nss});
+        for(my $idx = 0; $idx < @dns; $idx++) {
+            my $newNS = $dns[$idx];
+            if ( $idx <= $nNSS - 1) {
+                # There is a nameserver
+                $self->setNS($idx, $newNS);
+            } else {
+                # Add a new one
+                $resolverModel->add(nameserver => $newNS);
+            }
+        }
+
 }
 
 # Method: gateway
@@ -1509,70 +1613,108 @@ sub gateway
 
 # Method: routes
 #
-#   	Returns the configured routes
-# 
+#   	Return the configured static routes
+#
 # Returns:
-#   
-# 	array ref - each element contains a hash with keys 'network' and 
-# 	'gateway', where network is an IP block in CIDR format and gateway 
-# 	is an ip address.
+#
+# 	array ref - each element contains a hash ref with keys:
+#
+#          network - an IP block in CIDR format
+#          gateway - an IP address
+#
 sub routes
 {
-	my $self = shift;
-	#my @routes = @{$order->list};
-	#my @array = ();
-	#foreach (@routes) {
-	#	push(@array, $self->hash_from_dir($_));
-	#}
-	return $self->array_from_dir('routes');
-	#return \@array;
+#	my $self = shift;
+#	#my @routes = @{$order->list};
+#	#my @array = ();
+#	#foreach (@routes) {
+#	#	push(@array, $self->hash_from_dir($_));
+#	#}
+#	return $self->array_from_dir('routes');
+#	#return \@array;
+    my ($self) = @_;
+
+    my $staticRouteModel = $self->model('StaticRoute');
+    my @routes;
+    for my $row (@{$staticRouteModel->rows()}) {
+        push (@routes, { network => $row->printableValueByName('network'),
+                         gateway => $row->printableValueByName('gateway')});
+    }
+    return \@routes; 
+
 }
 
 # Method: addRoute
 #
-#   	Add a route
+#   	Add a static route
 #
 # Parameters:
 #
 #   	ip - the destination network (CIDR format)
 #   	mask - network mask
 #   	gateway - router for the given network
-#	 
-sub addRoute # (ip, mask, gateway) 
+#
+# Exceptions:
+#
+#       <EBox::Exceptions::DataExists> - thrown if the route does
+#       already exists
+#
+#       <EBox::Exceptions::External> - thrown if the gateway is not
+#       reachable
+#
+#sub addRoute # (ip, mask, gateway) 
+#{
+#	my ($self, $ip, $mask, $gw) = @_;
+#
+#	checkCIDR("$ip/$mask", __("network address"));
+#	checkIP($gw, __("ip address"));
+#	$self->gatewayReachable($gw, __("Gateway"));
+#
+#	$ip = ip_network($ip, mask_from_bits($mask));
+#	if ($self->_alreadyInRoute($ip, $mask)) {
+#		throw EBox::Exceptions::DataExists('data' => 'network route',
+#						  'value' => "$ip/$mask");
+#	}
+#
+#	my $id = $self->get_unique_id("r","routes");
+#
+#	$self->set_string("routes/$id/ip", $ip);
+#	$self->set_int("routes/$id/mask", $mask);
+#	$self->set_string("routes/$id/gateway", $gw);
+#}
+#
+# Method: gatewayDeleted
+#
+#    Mark an interface as changed for a route delete. The selected
+#    interface to be restarted must be the one which the gateway is
+#    in.
+#
+# Parameters:
+#
+#    gateway - String the gateway IP address
+#
+# Exceptions:
+#
+#    <EBox::Exceptions::MissingArgument> - thrown if any compulsory
+#    argument is missing
+#
+sub gatewayDeleted
 {
-	my ($self, $ip, $mask, $gw) = @_;
+    my ($self, $gw) = @_;
 
-	checkCIDR("$ip/$mask", __("network address"));
-	checkIP($gw, __("ip address"));
-	$self->_gwReachable($gw, __("Gateway"));
+    $gw or throw EBox::Exceptions::MissingArgument('gateway');
 
-	$ip = ip_network($ip, mask_from_bits($mask));
-	if ($self->_alreadyInRoute($ip, $mask)) {
-		throw EBox::Exceptions::DataExists('data' => 'network route',
-						  'value' => "$ip/$mask");
-	}
+    foreach my $iface (@{$self->allIfaces()}) {
+        my $host = $self->ifaceAddress($iface);
+        my $mask = $self->ifaceNetmask($iface);
+        my $meth = $self->ifaceMethod($iface);
+        (defined($meth) eq 'static') or next;
+        (defined($host) and defined($mask)) or next;
+        if (isIPInNetwork($host,$mask,$gw)) {
+            $self->_setChanged($iface);
+        }
+    }
 
-	my $id = $self->get_unique_id("r","routes");
-
-	$self->set_string("routes/$id/ip", $ip);
-	$self->set_int("routes/$id/mask", $mask);
-	$self->set_string("routes/$id/gateway", $gw);
-}
-
-sub _markIfaceForRoute # (gateway)
-{
-	my ($self, $gw) = @_;
-
-	foreach my $iface (@{$self->allIfaces()}) {
-		my $host = $self->ifaceAddress($iface);
-		my $mask = $self->ifaceNetmask($iface);
-		my $meth = $self->ifaceMethod($iface);
-		(defined($meth) eq 'static') or next;
-		(defined($host) and defined($mask)) or next;
-		if (isIPInNetwork($host,$mask,$gw)) {
-			$self->_setChanged($iface);
-		}
-	}
 }
 
 # Method: delRoute
@@ -1584,19 +1726,19 @@ sub _markIfaceForRoute # (gateway)
 #   	ip - the destination network 
 #   	mask - network mask
 #
-sub delRoute # (ip, mask) 
-{
-	my ($self, $ip, $mask) = @_;
-
-	my @routes = $self->all_dirs("routes");
-	foreach (@routes) {
-		($self->get_string("$_/ip") eq $ip) or next;
-		($self->get_int("$_/mask") eq $mask) or next;
-		$self->_markIfaceForRoute($self->get_string("$_/gateway"));
-		$self->delete_dir("$_");
-		return;
-	}
-}
+#sub delRoute # (ip, mask) 
+#{
+#	my ($self, $ip, $mask) = @_;
+#
+#	my @routes = $self->all_dirs("routes");
+#	foreach (@routes) {
+#		($self->get_string("$_/ip") eq $ip) or next;
+#		($self->get_int("$_/mask") eq $mask) or next;
+#		$self->gatewayDeleted($self->get_string("$_/gateway"));
+#		$self->delete_dir("$_");
+#		return;
+#	}
+#}
 
 #returns true if the interface has been marked as changed
 sub _hasChanged # (interface)
@@ -1654,6 +1796,7 @@ sub _setChanged # (interface)
 	}
 }
 
+# Generate the '/etc/resolv.conf' configuration file
 sub _generateResolver
 {
 	my $self = shift;
@@ -1671,11 +1814,18 @@ sub _generateResolver
 sub generateInterfaces
 {
 	my $self = shift;
-	my $file = EBox::Config::tmp . "/interfaces";
+	my $file = INTERFACES_FILE; 
+	my $tmpfile = EBox::Config::tmp . '/interfaces';
 	my $iflist = $self->allIfacesWithRemoved();
 
+	my $manager = new EBox::ServiceModule::Manager();
+	if ($manager->skipModification('network', $file)) {
+		EBox::info("Skipping modification of $file");
+		return;
+	}
+
 	#writing /etc/network/interfaces
-	open(IFACES, ">", $file) or
+	open(IFACES, ">", $tmpfile) or
 		throw EBox::Exceptions::Internal("Could not write on $file");
 	print IFACES "auto lo";
 	foreach (@{$iflist}) {
@@ -1684,6 +1834,7 @@ sub generateInterfaces
 			print IFACES " " . $_;
 		}
 	}
+	my ($gwIface, $gwIP) = $self->_defaultGwAndIface();
 	print IFACES "\niface lo inet loopback\n";
 	foreach my $ifname (@{$iflist}) {
 		my $method = $self->ifaceMethod($ifname);
@@ -1705,18 +1856,28 @@ sub generateInterfaces
 				"\n";
 			print IFACES "\tbroadcast " . 
 				$self->ifaceBroadcast($ifname) . "\n";
+            if (defined($gwIface) and defined($gwIP) and ($gwIface eq $ifname))
+            {
+                print IFACES "\tgateway $gwIP\n";
+            }
 		}
 	}
 	close(IFACES);
+
+	root("cp $tmpfile $file");
+	$manager->updateFileDigest('network', $file);
 }
 
+# Generate the static routes from routes() with "ip" command
 sub _generateRoutes
 {
-	my $self = shift;
-        my @routes = @{$self->routes};
+	my ($self) = @_;
+        # Delete those routes which are not useful anymore
+        $self->_removeRoutes();
+        my @routes = @{$self->routes()};
         (@routes) or return;
 	foreach (@routes) {
-		my $net = $_->{ip} . "/" . $_->{mask};
+		my $net = $_->{network};
 		my $router = $_->{gateway};
 		if (route_is_up($net, $router)) {
 			root("/sbin/ip route del $net via $router");
@@ -1725,6 +1886,29 @@ sub _generateRoutes
 	}
 }
 
+# Remove those static routes which user has marked as deleted
+sub _removeRoutes
+{
+    my ($self) = @_;
+
+    my $deletedModel = $self->model('DeletedStaticRoute');
+    foreach my $row (@{$deletedModel->rows()}) {
+        my $network = $row->elementByName('network')->printableValue();
+        my $gateway = $row->elementByName('gateway')->printableValue();
+        if ( route_is_up($network, $gateway)) {
+            root("/sbin/ip route del $network via $gateway");
+        }
+        # Perform deletion in two phases to let eBox perform sync correctly
+        if ( $row->elementByName('deleted')->value() ) {
+            $deletedModel->removeRow($row->id(), 1);
+        } else {
+            $row->elementByName('deleted')->setValue(1);
+            $row->storeElementByName('deleted');
+        }
+    }
+
+}
+
 sub _multigwRoutes
 {
 	my $self = shift;
@@ -1827,7 +2011,7 @@ sub _regenConfig
 
 	my $gateway = $self->gateway;
 	my $skipdns = undef;
-	my $file = EBox::Config::tmp . "/interfaces";
+	my $file = INTERFACES_FILE;
 
 	try {
 		root("/sbin/modprobe 8021q");
@@ -1837,12 +2021,6 @@ sub _regenConfig
 	} catch EBox::Exceptions::Internal with {};
 
 	$self->DHCPGatewayCleanUpFix();
-	my $dhcpgw = $self->DHCPGateway();
-	unless ($dhcpgw and ($dhcpgw ne '')) {
-		try {
-			root("/sbin/ip route del default table default");
-		} catch EBox::Exceptions::Internal with {};
-	}
 
 	#bring down changed interfaces
 	my $iflist = $self->allIfacesWithRemoved();
@@ -1900,6 +2078,18 @@ sub _regenConfig
 		}
 	}
 
+	my $dhcpgw = $self->DHCPGateway();
+	unless ($dhcpgw and ($dhcpgw ne '')) {
+		try {
+			root("/sbin/ip route del default table default");
+		} catch EBox::Exceptions::Internal with {};
+		try {
+			root("/sbin/ip route del default");
+		} catch EBox::Exceptions::Internal with {};
+	
+	}
+
+
 
 	my $multipathCmd = $self->_multipathCommand();
 	if ($gateway) {
@@ -1917,15 +2107,21 @@ sub _regenConfig
 	$self->_multigwRoutes();
 	$self->_cleanupVlanIfaces();
 
-	# regenerate config for the bit rate report
-	EBox::Network::Report::ByteRate->_regenConfig();
+	# XXX uncomment when DynLoader bug with locales is fixed
+# 	# regenerate config for the bit rate report
+# 	EBox::Network::Report::ByteRate->_regenConfig();
 }
 
 sub stopService
 {
 	my $self = shift;
 
-	my $file = EBox::Config::tmp . "/interfaces";
+	# XXX uncomment when DynLoader bug with locales is fixed
+	# EBox::Network::Report::ByteRate->stopService();
+
+	return unless ($self->configured());
+
+	my $file = INTERFACES_FILE;
 	my $iflist = $self->allIfaces();
 	foreach my $if (@{$iflist}) {
 		try {
@@ -1935,7 +2131,9 @@ sub stopService
 		} catch EBox::Exceptions::Internal with {};
 	}
 
-	EBox::Network::Report::ByteRate->stopService();
+# XXX uncomment when DynLoader bug with locales is fixed
+#	EBox::Network::Report::ByteRate->stopService();
+
 }
 
 #internal use functions
@@ -2009,47 +2207,74 @@ sub _routersReachableIfChange # (interface, newaddress?, newmask?)
 	return 1;
 }
 
-sub _gwReachable # (address, name?)
+# Method: gatewayReachable
+#
+#       Check if a given gateway address is reachable with the current
+#       network configuration
+#
+# Parameters:
+#
+#       gw - String the IP address for the gateway
+#
+#       name - String A name to be shown if exception is launched. If
+#       no given, then an exception is not launched. *(Optional)*
+#       Default value: undef
+#
+# Returns:
+#
+#       Boolean - if name is not present, indicate whether the given
+#       gateway is reachable or not
+#
+# Exceptions:
+#
+#       <EBox::Exceptions::MissingArgument> - thrown if any compulsory
+#       argument is missing
+#
+#       <EBox::Exceptions::External> - thrown if name is supplied and
+#       the gateway is not reachable
+#
+sub gatewayReachable
 {
-	my $self = shift;
-	my $gw   = shift;
-	my $name = shift;
+    my ($self, $gw, $name) = @_;
 
-	my $reachableByNoStaticIface = undef;
+    $gw or throw EBox::Exceptions::MissingArgument('gw');
 
-	my $cidr_gw = "$gw/32";
-	foreach my $iface (@{$self->allIfaces()}) {
-		my $host = $self->ifaceAddress($iface);
-		my $mask = $self->ifaceNetmask($iface);
+    my $reachableByNoStaticIface = undef;
 
-		(defined($host) and defined($mask)) or next;
+    my $cidr_gw = "$gw/32";
+    foreach my $iface (@{$self->allIfaces()}) {
+        my $host = $self->ifaceAddress($iface);
+        my $mask = $self->ifaceNetmask($iface);
 
-		checkIPNetmask($gw, $mask) or next;
+        (defined($host) and defined($mask)) or next;
 
-		if (isIPInNetwork($host,$mask,$cidr_gw)) {
-		  my $meth = $self->ifaceMethod($iface);
-		  if ($meth ne 'static') {
-		    $reachableByNoStaticIface = $iface;
-		    next; 
-		  }
+        checkIPNetmask($gw, $mask) or next;
 
-		  return 1;
-		}
-	}
+        if (isIPInNetwork($host,$mask,$cidr_gw)) {
+            my $meth = $self->ifaceMethod($iface);
+            if ($meth ne 'static') {
+                $reachableByNoStaticIface = $iface;
+                next; 
+            }
 
-	if ($name) {
-	  if (not $reachableByNoStaticIface) {
+            return 1;
+        }
+    }
+
+    if ($name) {
+        if (not $reachableByNoStaticIface) {
 	    throw EBox::Exceptions::External(
-					     __x("Gateway {gw} not reachable", gw => $gw));
-	  }
-	  else {
-	    throw EBox::Exceptions::External(
-					     __x("Gateway {gw} must be reacheable by a static interface. Currently is reacheable by {iface} which is not static", gw => $gw, iface => $reachableByNoStaticIface) );
-	  }
-
+                __x("Gateway {gw} not reachable", gw => $gw));
         } else {
-		return undef;
-	}
+	    throw EBox::Exceptions::External(
+                __x("Gateway {gw} must be reachable by a static interface. "
+                    . "Currently it is reachable by {iface} which is not static",
+                    gw => $gw, iface => $reachableByNoStaticIface) );
+        }
+
+    } else {
+        return undef;
+    }
 }
 
 sub _alreadyInRoute # (ip, mask) 
@@ -2354,10 +2579,13 @@ sub summary
 		}
 	}
 	$composite->add($item);
-        my $monSummary = EBox::Network::Report::ByteRate->summary();
-        if ( defined($monSummary) ) {
-            $composite->add($monSummary);
-        }
+
+	
+# XXX uncomment when DynLoader bug with locales is fixed
+#         my $monSummary = EBox::Network::Report::ByteRate->summary();
+#         if ( defined($monSummary) ) {
+#             $composite->add($monSummary);
+#         }
 
         return $composite;
 }
@@ -2376,9 +2604,9 @@ sub menu
 
 	$folder->add(new EBox::Menu::Item('url' => 'Network/Ifaces',
 					  'text' => __('Interfaces')));
-	$folder->add(new EBox::Menu::Item('url' => 'Network/DNS',
+	$folder->add(new EBox::Menu::Item('url' => 'Network/View/DNSResolver',
 					  'text' => 'DNS'));
-	$folder->add(new EBox::Menu::Item('url' => 'Network/Routes',
+	$folder->add(new EBox::Menu::Item('url' => 'Network/View/StaticRoute',
 					  'text' => __('Routes')));
 	$folder->add(new EBox::Menu::Item('url' => 'Network/Diag',
 					  'text' => __('Diagnosis')));
@@ -2388,9 +2616,12 @@ sub menu
 	$folder->add(new EBox::Menu::Item('url' => 
 						'Network/View/MultiGwRulesDataTable',
 					  'text' => __('Balance traffic')));
-	$folder->add(new EBox::Menu::Item('url' => 
-						'Network/Composite/ByteRate',
-					  'text' => __('Traffic rate monitor')));
+
+
+# XXX uncomment when DynLoader bug with locales is fixed
+# 	$folder->add(new EBox::Menu::Item('url' => 
+# 						'Network/Composite/ByteRate',
+# 					  'text' => __('Traffic rate monitor')));
 
 
 
@@ -2457,6 +2688,19 @@ sub gateways
 
 }
 
+sub _defaultGwAndIface
+{
+	my ($self) = @_;
+
+	my $row = $self->gatewayModel()->find('default' => 1);
+
+	if ($row) {
+		return ($row->{'interface'}, $row->{'ip'});
+	} else {
+		return (undef, undef);
+	}
+}
+
 
 
 # Method: gatewaysWithMac
